@@ -55,6 +55,19 @@ liTemplate.remove();
 
 document.querySelector("#info").addEventListener("click", e => {
 	let button = e.target;
+  if (button.tagName === "A") {
+    setTimeout(close, 100);
+    return;
+  }
+  if (button.matches(".toggle-source")) {
+    let parent = button.parentNode;
+    if (!parent.querySelector(".source").textContent) {
+      parent.querySelector("a").click();
+    } else {
+      parent.classList.toggle("visible");
+    }
+    return;
+  }
 	if (!button.matches(".buttons > button")) return;
 	let li = button.closest("li");
 	let entry = li && li._scriptEntry || [currentReport.url, "Page's site"];
@@ -111,11 +124,21 @@ function createList(data, group){
    container.classList.add("empty");
  }
  // generate list
+ let viewSourceToHuman = /^view-source:(.*)#line(\d+)\(([^)]*)\)[^]*/;
  for (let entry of entries) {
    let [scriptId, reason] = entry;
 	 let li = liTemplate.cloneNode(true);
 	 let a = li.querySelector("a");
 	 a.href = scriptId.split("(")[0];
+   if (scriptId.startsWith("view-source:")) {
+     a.target ="LibreJS-ViewSource";
+     let source = scriptId.match(/\n([^]*)/);
+     if (source)  {
+       li.querySelector(".source").textContent = source[1];
+       li.querySelector(".toggle-source").style.display = "inline";
+     }
+     scriptId = scriptId.replace(viewSourceToHuman, "$3 at line $2 of $1");
+   }
    a.textContent = scriptId;
 	 li.querySelector(".reason").textContent = reason;
    let bySite = !!reason.match(/https?:\/\/[^/]+\/\*/);
