@@ -22,10 +22,9 @@
 
 var acorn = require('acorn');
 var acornLoose = require('acorn-loose');
-var jssha = require('jssha');
 var legacy_license_lib = require("./legacy_license_check.js");
 var {ResponseProcessor} = require("./bg/ResponseProcessor");
-var {Storage, ListStore} = require("./common/Storage");
+var {Storage, ListStore, hash} = require("./common/Storage");
 var {ListManager} = require("./bg/ListManager");
 var {ExternalLicenses} = require("./bg/ExternalLicenses");
 
@@ -51,16 +50,6 @@ function dbg_print(a,b){
 	}
 }
 
-/**
-*	Wrapper around crypto lib
-*
-*/
-function hash(source){
-	var shaObj = new jssha("SHA-256","TEXT")
-	shaObj.update(source);
-	return shaObj.getHash("HEX");
-}
-
 /*
 	NONTRIVIAL THINGS:
 	- Fetch
@@ -214,7 +203,7 @@ async function updateReport(tabId, oldReport, updateUI = false){
 		}
 	}
 	activityReports[tabId] = newReport;
-	browser.sessions.setTabValue(tabId, url, newReport);
+	if (browser.sessions) browser.sessions.setTabValue(tabId, url, newReport);
 	dbg_print(newReport);
 	if (updateUI && activeMessagePorts[tabId]) {
 		dbg_print(`[TABID: ${tabId}] Sending script blocking report directly to browser action.`);
@@ -290,7 +279,7 @@ async function addReportEntry(tabId, scriptHashOrUrl, action) {
 		}
 	}
 
-	browser.sessions.setTabValue(tabId, report.url, report);
+	if (browser.sessions) browser.sessions.setTabValue(tabId, report.url, report);
 	updateBadge(tabId, report);
 	return entryType;
 }
@@ -336,7 +325,11 @@ async function connected(p) {
 		for (let action of ["whitelist", "blacklist", "forget"]) {
 			if (m[action]) {
 				let [key] = m[action];
-				if (m.site) key = ListStore.siteItem(key);
+				if (m.site) {
+					key = ListStore.siteItem(key);
+				} else {
+					key = ListStore.inlineItem(key) || key;
+				}
 				await listManager[action](key);
 				update = true;
 			}
@@ -419,7 +412,8 @@ async function onTabUpdated(tabId, changedInfo, tab) {
 	let [url] = tab.url.split("#");
 	let report = activityReports[tabId];
 	if (!(report && report.url === url)) {
-		let cache = await browser.sessions.getTabValue(tabId, url);
+		let cache = browser.sessions &&
+			await browser.sessions.getTabValue(tabId, url) || null;
 		// on session restore tabIds may change
 		if (cache && cache.tabId !== tabId) cache.tabId = tabId;
 		updateBadge(tabId, activityReports[tabId] = cache);
@@ -775,23 +769,20 @@ async function get_script(response, url, tabId = -1, whitelisted = false, index
 	let report = activityReports[tabId] || (activityReports[tabId] = await createReport({tabId}));
 	updateBadge(tabId, report, !verdict);
 	let category = await addReportEntry(tabId, sourceHash, {"url": domain, [verdict ? "accepted" : "blocked"]: [url, reason]});
-	let scriptSource = verdict ? response : editedSource;
 	switch(category) {
 		case "blacklisted":
-			if (response.startsWith("javascript:"))
-				return result(`# LibreJS: script ${category} by user.`);
-			else
-				return result(`/* LibreJS: script ${category} by user. */`);
+			editedSource = `/* LibreJS: script ${category} by user. */`;
+			return result(response.startsWith("javascript:")
+				? `javascript:void(${encodeURIComponent(editedSource)})` : editedSource);
 		case "whitelisted":
-			if (response.startsWith("javascript:"))
-				return result(scriptSource);
-			else
-				return result(`/* LibreJS: script ${category} by user. */\n${scriptSource}`);
+			return result(response.startsWith("javascript:")
+				? response : `/* LibreJS: script ${category} by user. */\n${response}`);
 		default:
-                        if (response.startsWith("javascript:"))
-				return result(scriptSource);
-			else
-				return result(`/* LibreJS: script ${category}. */\n${scriptSource}`);
+			let scriptSource = verdict ? response : editedSource;
+      return result(response.startsWith("javascript:")
+				? (verdict ? scriptSource : `javascript:void(/* ${scriptSource} */)`)
+				: `/* LibreJS: script ${category}. */\n${scriptSource}`
+			);
 	}
 }
 
@@ -800,8 +791,14 @@ function updateBadge(tabId, report = null, forceRed = false) {
 	let blockedCount = report ? report.blocked.length + report.blacklisted.length : 0;
 	let [text, color] = blockedCount > 0 || forceRed
 		? [blockedCount && blockedCount.toString() || "!" , "red"] : ["✓", "green"]
-	browser.browserAction.setBadgeText({text, tabId});
-	browser.browserAction.setBadgeBackgroundColor({color, tabId});
+	let {browserAction} = browser;
+	if ("setBadgeText" in browserAction) {
+		browserAction.setBadgeText({text, tabId});
+		browserAction.setBadgeBackgroundColor({color, tabId});
+	} else {
+		// Mobile
+		browserAction.setTitle({title: `LibreJS (${text})`, tabId});
+	}
 }
 
 /**
@@ -1032,7 +1029,7 @@ async function editHtml(html, documentUrl, tabId, frameId, whitelisted){
 	// get the potential inline source that can contain a license
 	for (let script of scripts) {
 		// The script must be in-line and exist
-		if(script && !script.src){
+		if(script && !script.src) {
 			first_script_src = script.textContent;
 			break;
 		}
@@ -1042,30 +1039,52 @@ async function editHtml(html, documentUrl, tabId, frameId, whitelisted){
 	if (first_script_src != "") {
 		license = legacy_license_lib.check(first_script_src);
 	}
+
+	let findLine = finder => finder.test(html) && html.substring(0, finder.lastIndex).split(/\n/).length || 0;
 	if (read_metadata(meta_element) || license) {
 		console.log("Valid license for intrinsic events found");
-		addReportEntry(tabId, url, {url, "accepted":[url, `Global license for the page: ${license}`]});
+		let line, extras;
+		if (meta_element) {
+		  line = findLine(/id\s*=\s*['"]?LibreJS-info\b/gi);
+			extras = "(0)";
+		} else if (license) {
+			line = html.substring(0, html.indexOf(first_script_src)).split(/\n/).length;
+			extras = "\n" + first_script_src;
+		}
+		let viewUrl = line ? `view-source:${documentUrl}#line${line}(<${meta_element ? meta_element.tagName : "SCRIPT"}>)${extras}` : url;
+		addReportEntry(tabId, url, {url, "accepted":[viewUrl, `Global license for the page: ${license}`]});
 		// Do not process inline scripts
 		scripts = [];
 	} else {
+		let dejaVu = new Map(); // deduplication map & edited script cache
 		let modified = false;
-
 		// Deal with intrinsic events
 		let intrinsecindex = 0;
+		let intrinsicFinder = /<[a-z][^>]*\b(on\w+|href\s*=\s*['"]?javascript:)/gi;
 		for (let element of html_doc.all) {
-			for (let attr of element.attributes){
-				if (attr.name.startsWith("on") || (attr.name === "href" && attr.value.startsWith("javascript:"))){
+			let line = -1;
+			for (let attr of element.attributes) {
+				let {name, value} = attr;
+				value = value.trim();
+				if (name.startsWith("on") || (name === "href" && value.toLowerCase().startsWith("javascript:"))){
 					intrinsecindex++;
+					if (line === -1) {
+						line = findLine(intrinsicFinder);
+					}
 					try {
-						let url = `${documentUrl}# Intrinsic event ${intrinsecindex} [${attr.name}]`;
-						let edited = await get_script(attr.value, url, tabId, whitelist.contains(url));
-							if (edited) {
-								let value = edited;
-								if (value !== attr.value) {
-									modified = true;
-									attr.value = value;
-								}
-							}
+						let key = `<${element.tagName} ${name}="${value}">`;
+						let edited;
+						if (dejaVu.has(key)) {
+							edited = dejaVu.get(key);
+						} else {
+							let url = `view-source:${documentUrl}#line${line}(<${element.tagName} ${name}>)\n${value.trim()}`;
+							if (name === "href") value = decodeURIComponent(value);
+							edited = await get_script(value, url, tabId, whitelist.contains(url));						dejaVu.set(key, edited);
+						}
+						if (edited && edited !== value) {
+							modified = true;
+							attr.value = edited;
+						}
 					} catch (e) {
 						console.error(e);
 					}
@@ -1074,16 +1093,24 @@ async function editHtml(html, documentUrl, tabId, frameId, whitelisted){
 		}
 
 		let modifiedInline = false;
+		let scriptFinder = /<script\b/ig;
 		for(let i = 0, len = scripts.length; i < len; i++) {
 			let script = scripts[i];
-			let url = `${documentUrl}# script ${i}`;
+			let line = findLine(scriptFinder);
 			if (!script.src && !(script.type && script.type !== "text/javascript")) {
-				let edited = await get_script(script.textContent, url, tabId, whitelisted, i);
-				if (edited) {
-					let edited_source = edited[0];
-					let unedited_source = script.textContent.trim();
-					if (edited_source.trim() !== unedited_source) {
-						script.textContent = edited_source;
+				let source = script.textContent.trim();
+				let editedSource;
+				if (dejaVu.has(source)) {
+					editedSource = dejaVu.get(source);
+				} else {
+					let url = `view-source:${documentUrl}#line${line}(<SCRIPT>)\n${source}`;
+					let edited = await get_script(source, url, tabId, whitelisted, i);
+					editedSource = edited && edited[0].trim();
+					dejaVu.set(url, editedSource);
+				}
+				if (editedSource) {
+					if (source !== editedSource) {
+						script.textContent = editedSource;
 						modified = modifiedInline = true;
 					}
 				}
